#!/usr/bin/env python
import sys
import time
import thread
# from pydbgr.api import debug

def myfunction(string,sleeptime,lock,*args):
    while 1:
	#entering critical section
        # debug()
        lock.acquire() 
        print string," Now Sleeping after Lock acquired for ",sleeptime
        time.sleep(sleeptime) 
        print string," Now releasing lock and then sleeping again"
        lock.release()
	#exiting critical section
        time.sleep(sleeptime) # why?

if __name__=="__main__":
    lock=thread.allocate_lock()
    thread.start_new_thread(myfunction,("Thread No:1",1,lock))
    thread.start_new_thread(myfunction,("Thread No:2",1,lock))

    while 1:pass




# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2010, 2012 Rocky Bernstein
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
#    02110-1301 USA.

from import_relative import *
# Our local modules

Mbase_subcmd  = import_relative('base_subcmd', '..', 'pydbgr')

class ShowMaxString(Mbase_subcmd.DebuggerShowIntSubcommand):
    "Show maximum string length to use in string-oriented output"
    min_abbrev = 2
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'Unit test for pydbgr.io.input'
import os, unittest
from import_relative import import_relative, get_srcdir

Minput = import_relative('pydbgr.io.input', '...')

class TestDebuggerInput(unittest.TestCase):
    
    def test_DebuggerInput(self):
        cmdhelper_file=os.path.join(get_srcdir(),'cmdhelper.py') 
        inp = Minput.DebuggerUserInput(cmdhelper_file)
        self.assertTrue(inp, 'Should have gotten a DebuggerInput object back')
        line = inp.readline()
        self.assertEqual('# -*- coding: utf-8 -*-', line)
        inp.close()
        # Should be okay
        inp.close() 
        return

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#  Copyright (C) 2009 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from import_relative import import_relative
# Our local modules

Mbase_subcmd  = import_relative('base_subcmd', '..', 'pydbgr')

class ShowTrace(Mbase_subcmd.DebuggerShowBoolSubcommand):
    """Show event tracing.

See also 'set events'."""

    min_abbrev = 2 # Need at least "show tr"




# -*- coding: utf-8 -*-
""" Mock classes used for testing which in turn really come from 
pydbgr.processor.command.mock """

from import_relative import import_relative

default   = import_relative('lib.default', '...pydbgr') # Default settings
mock      = import_relative('processor.command.mock', '...pydbgr')

dbg_setup = mock.dbg_setup



#!/usr/bin/env python
'Unit test for pydbgr.lib.file'
import os, stat, tempfile, unittest
from import_relative import import_relative

import_relative('lib', '...pydbgr')
Mfile = import_relative('lib.file', '...pydbgr')

class TestLibFile(unittest.TestCase):

    def test_lookupmodule(self):
        m, f = Mfile.lookupmodule('os.path')
        self.assertTrue(f)
        self.assertTrue(m)
        m, f = Mfile.lookupmodule(__file__)
        self.assertTrue(f)
        self.assertEqual(None, m)
        self.assertEqual((None, None), Mfile.lookupmodule('fafdsafdsa'))
        return

    def test_pyc2py(self):
        """Test clifns.pyc2py()"""
        self.assertEqual('foo.py', Mfile.file_pyc2py("foo.pyc"))
        fn = 'stays-the-same.py'
        self.assertEqual(fn , Mfile.file_pyc2py(fn))
        fn = 'stays-the-same-without-suffix'
        self.assertEqual(fn , Mfile.file_pyc2py(fn))
        return

    def test_readable(self):
        self.assertFalse(Mfile.readable('fdafdsa'))
        for mode, can_read in [(stat.S_IRUSR, True), (stat.S_IWUSR, False)]: 
            f = tempfile.NamedTemporaryFile()
            os.chmod(f.name, mode)
            self.assertEqual(can_read, Mfile.readable(f.name))
            f.close()
            pass
        return
    
    pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2009 Rocky Bernstein
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from import_relative import import_relative
Mbase_subcmd  = import_relative('base_subcmd', '..', 'pydbgr')

class ShowAutoPython(Mbase_subcmd.DebuggerShowBoolSubcommand):
    '''**show autopython**

Show whether we go into a python shell when automatically when the
debugger is entered.

Change with **set autopython**
'''
    short_help = "Show automatic Python shell entry"
    min_abbrev = len('autopy')
    pass

if __name__ == '__main__':
    Mhelper = import_relative('__demo_helper__', '.', 'pydbgr')
    mgr = Mhelper.demo_run(ShowAutoPython)
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2013 Rocky Bernstein
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os, threading
from import_relative import import_relative

# Our local modules
Mbase_cmd  = import_relative('base_cmd', top_name='pydbgr')
Mexcept    = import_relative('exception', '...', 'pydbgr')

class QuitCommand(Mbase_cmd.DebuggerCommand):
    """**quit** [**unconditionally**]

Gently terminate the debugged program.

The program being debugged is aborted via a *DebuggerQuit*
exception.

When the debugger from the outside (e.g. via a `pydbgr` command), the
debugged program is contained inside a try block which handles the
*DebuggerQuit* exception.  However if you called the debugger was
started in the middle of a program, there might not be such an
exception handler; the debugged program still terminates but generally
with a traceback showing that exception.

If the debugged program is threaded or worse threaded and deadlocked,
raising an exception in one thread isn't going to quit the
program. For this see `exit` or `kill` for more forceful termination
commands.

Also, see `run` and `restart` for other ways to restart the debugged
program.
"""

    aliases       = ('q',)
    category      = 'support'
    min_args      = 0
    max_args      = 0
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = False
    short_help    = 'Terminate the program - gently'

    def nothread_quit(self, arg):
        """ quit command when there's just one thread. """

        self.debugger.core.stop()
        self.debugger.core.execution_status = 'Quit command'
        raise Mexcept.DebuggerQuit

    def threaded_quit(self, arg):
        """ quit command when several threads are involved. """
        self.msg("Quit for threading not fully done yet. Try 'kill'.")
        return False

    def run(self, args):
        threading_list = threading.enumerate()
        if (len(threading_list) == 1 and
            threading_list[0].getName() == 'MainThread'):
            # We just have a main thread so that's safe to quit
            return self.nothread_quit(args)
        else:
            return self.threaded_quit(args)
        pass

if __name__ == '__main__':
    mock = import_relative('mock')
    Mdebugger = import_relative('debugger', '...')
    d, cp = mock.dbg_setup()
    command = QuitCommand(cp)
    try: 
        command.run(['quit'])
    except Mexcept.DebuggerQuit:
        print("A got 'quit' a exception. Ok, be that way - I'm going home.")
        pass

    class MyThread(threading.Thread):
        def run(self):
            command.run(['quit'])
            return
        pass

    t = MyThread()
    t.start()
    t.join()
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2009, 2013 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''Some singleton debugger methods that can be called without first
creating a debugger object -- these methods will create a debugger object, 
if necessary, first.
'''

# The following routines could be done via something like the following
# untested code:
# def _build_standalone(methodname, docstring=None):
#     def function(arg, arg2=None, globals=None, locals=None):
#         Debugger().getattr(methodname)(arg, arg2, globals, locals)
#        return 
#    function.__name__ = methodname
#    function.__doc__ = docstring
#    return function
#
# But this a bit cumbersome and perhaps overkill for the 2 or so
# functions below.  (It also doesn't work once we add the exception handling
# we see below. So for now, we'll live with the code duplication.

import sys
from import_relative import import_relative

Mdebugger    = import_relative('debugger', top_name='pydbgr')
Mpost_mortem = import_relative('post_mortem', top_name='pydbgr')

def debugger_on_post_mortem():
    '''Call debugger on an exeception that terminates a program'''
    sys.excepthook = Mpost_mortem.post_mortem_excepthook
    return

def run_eval(expression, debug_opts=None, start_opts=None, globals_=None, 
             locals_=None):

    """Evaluate the expression (given as a string) under debugger
    control starting with the statement subsequent to the place that
    this appears in your program.

    This is a wrapper to Debugger.run_eval(), so see that.

    When run_eval() returns, it returns the value of the expression.
    Otherwise this function is similar to run()."""

    
    dbg = Mdebugger.Debugger(opts=debug_opts)
    try:
        return dbg.run_eval(expression, start_opts=start_opts, 
                            globals_=globals_, locals_=locals_)
    except:
        Mpost_mortem.uncaught_exception(dbg)
        pass
    return

def run_call(func, debug_opts=None, start_opts=None, *args, **kwds):

    """Call the function (a function or method object, not a string)
    with the given arguments starting with the statement subsequent to
    the place that this appears in your program.

    When run_call() returns, it returns whatever the function call
    returned.  The debugger prompt appears as soon as the function is
    entered."""

    dbg = Mdebugger.Debugger(opts=debug_opts) 
    try:
        return dbg.run_call(func, start_opts, *args, **kwds)
    except:
        Mpost_mortem.uncaught_exception(dbg)
        pass
    return

def run_exec(statement, debug_opts=None, start_opts=None, globals_=None, 
             locals_=None):

    """Execute the statement (given as a string) under debugger
    control starting with the statement subsequent to the place that
    this run_call appears in your program.

    This is a wrapper to Debugger.run_exec(), so see that.

    The debugger prompt appears before any code is executed;
    you can set breakpoints and type 'continue', or you can step
    through the statement using 'step' or 'next'

    The optional globals_ and locals_ arguments specify the environment
    in which the code is executed; by default the dictionary of the
    module __main__ is used."""

    dbg = Mdebugger.Debugger(opts=debug_opts) 
    try:
        return dbg.run_exec(statement, start_opts=start_opts, 
                            globals_=globals_, locals_=locals_)
    except:
        Mpost_mortem.uncaught_exception(dbg)
        pass
    return

def debug(dbg_opts=None, start_opts=None, post_mortem=True,
          step_ignore=1):
    """ 
Enter the debugger. Use like this:

    ... # Possibly some Python code
    import pydbgr.api # Needed only once
    ... # Possibly some more Python code
    pydbgr.api.debug() # You can wrap inside conditional logic too
    pass  # Stop will be here.
    # Below is code you want to use the debugger to do things.
    ....  # more Python code
    # If you get to a place in the program where you aren't going 
    # want to debug any more, but want to remove debugger trace overhead:
    pydbgr.api.stop() 

In situations where you want an immediate stop in the "debug" call
rather than the statement following it ("pass" above), add parameter
step_ignore=0 to debug() like this:

    import pydbgr.api  # Needed only once
    # ... as before
    pydbgr.api.debug(step_ignore=0)
    # ... as before


Module variable debugger_obj from module pydbgr.debugger is used as
the debugger instance variable; it can be subsequenly used to change
settings or alter behavior. It should be of type Debugger (found in
module pydbgr). If not, it will get changed to that type.

   Example:
   $ python
   >>> from pydbgr.debugger import debugger_obj
   >>> type(debugger_obj)
   <type 'NoneType'>
   >>>  import pydbgr.api 
   >>>  pydbgr.api.debug()
   ...
   (Pydbgr) c
   >>> from pydbgr.debugger import debugger_obj
   >>> debugger_obj
   <pydbgr.debugger.Debugger instance at 0x7fbcacd514d0>
   >>>

If however you want your own separate debugger instance, you can
create it from the debugger class Debugger() from module
pydbgr.debugger.

Example:
  $ python
  >>> from pydbgr.debugger import Debugger
  >>> dbgr = Debugger()  # Add options as desired
  >>> dbgr
  <pydbgr.debugger.Debugger instance at 0x2e25320>

`dbg_opts' is an optional "options" dictionary that gets fed
pydbgr.Debugger(); `start_opts' are the optional "options"
dictionary that gets fed to pydbgr.Debugger.core.start().

Parameter "step_ignore" specifies how many line events to ignore after the
debug() call. 0 means don't even wait for the debug() call to finish.
"""
    if Mdebugger.Debugger != type(Mdebugger.debugger_obj):
        Mdebugger.debugger_obj = Mdebugger.Debugger(dbg_opts)
        Mdebugger.debugger_obj.core.add_ignore(debug, stop)
        pass
    core = Mdebugger.debugger_obj.core
    frame = sys._getframe(0)
    core.set_next(frame)
    if not core.is_started():
        core.start(start_opts)
        pass
    if post_mortem:
        debugger_on_post_mortem()
        pass
    if 0 == step_ignore:
        frame                   = sys._getframe(1)
        core.stop_reason        = 'at a debug() call'
        old_trace_hook_suspend  = core.trace_hook_suspend
        core.trace_hook_suspend = True
        core.processor.event_processor(frame, 'line', None)
        core.trace_hook_suspend = old_trace_hook_suspend
    else:
        core.step_ignore = step_ignore-1;
        pass
    return 

def stop(opts=None):
    if Mdebugger.Debugger == type(Mdebugger.debugger_obj):
        return Mdebugger.debugger_obj.stop(opts)
    return None

# Demo it
if __name__=='__main__':
    import sys, tracer
    def foo(n):
        y = n
        for i in range(n):
            print(i)
            pass
        return y
    Mdefault = import_relative('default', 'lib', 'pydbgr')
    settings = dict(Mdefault.DEBUGGER_SETTINGS)
    settings.update({'trace': True, 'printset': tracer.ALL_EVENTS})
    debug_opts={'step_ignore': -1, 'settings': settings}
    print('Issuing: run_eval("1+2")')
    print(run_eval('1+2', debug_opts=debug_opts))
    print('Issuing: run_exec("x=1; y=2")')
    run_exec('x=1; y=2', debug_opts=debug_opts)
    print('Issuing: run_call(foo, debug_opts, None, 2)')
    run_call(foo, debug_opts, None, 2)
    # if len(sys.argv) > 1:
    #     # FIXME: should this work better?
    #     # print 'Issuing interactive: run_exec(x=1; y=2)'
    #     # run_exec('x=1; y=2')
    #     print 'Issuing interactive: run_call(foo)'
    #     run_call(foo, debug_opts)
    pass
 



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2010, 2013 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Debugger class and top-level debugger functions.

This module contains the `Debugger' class and some top-level routines
for creating and invoking a debugger. Most of this module serves as:
  * a wrapper for `Debugger.core' routines, 
  * a place to define `Debugger' exceptions, and 
  * `Debugger' settings.

See also module `cli' which contains a command-line interface to debug
a Python script and `core' which contains the core debugging
start/stop and event-handling dispatcher and `client.py' which is a
user or client-side code for connecting to server'd debugged program.
"""

# Our local modules
from import_relative import import_relative

import_relative('processor', '..pydbgr')

Mcore   = import_relative('core', '.lib', 'pydbgr')
Mexcept = import_relative('exception', '.', 'pydbgr')

# Default settings used here
Mdefault  = import_relative('default', '.lib', 'pydbgr') 

Muser     = import_relative('user', '.interfaces', 'pydbgr')
Mmisc     = import_relative('misc', top_name='pydbgr')
Msig      = import_relative('sighandler', '.lib', 'pydbgr')

# Common Python packages
import sys, types

# External Egg packages
import tracer, tracefilter

debugger_obj = None

class Debugger:

    # The following functions have to be defined before DEFAULT_INIT_OPTS which 
    # includes references to these.

    # FIXME DRY run, run_exec, run_eval.

    def run(self, cmd, start_opts=None, globals_=None, locals_=None):
        """ Run debugger on string `cmd' using builtin function eval
        and if that builtin exec.  Arguments `globals_' and `locals_'
        are the dictionaries to use for local and global variables. By
        default, the value of globals is globals(), the current global
        variables. If `locals_' is not given, it becomes a copy of
        `globals_'.

        Debugger.core.start settings are passed via optional
        dictionary `start_opts'. Overall debugger settings are in
        Debugger.settings which changed after an instance is created
        . Also see `run_eval' if what you want to run is an
        run_eval'able expression have that result returned and
        `run_call' if you want to debug function run_call.
        """
        if globals_ is None:
            globals_ = globals()
        if locals_ is None:
            locals_ = globals_
        if not isinstance(cmd, types.CodeType):
            self.eval_string = cmd
            cmd = cmd+'\n'
            pass
        retval = None
        self.core.start(start_opts)
        try:
            retval = eval(cmd, globals_, locals_)
        except SyntaxError:
            try:
                exec(cmd, globals_, locals_)
            except Mexcept.DebuggerQuit:
                pass
            except Mexcept.DebuggerQuit:
                pass
            pass
        except Mexcept.DebuggerQuit:
            pass
        finally:
            self.core.stop()
        return retval

    def run_exec(self, cmd, start_opts=None, globals_=None, locals_=None):
        """ Run debugger on string `cmd' which will executed via the
        builtin function exec. Arguments `globals_' and `locals_' are
        the dictionaries to use for local and global variables. By
        default, the value of globals is globals(), the current global
        variables. If `locals_' is not given, it becomes a copy of
        `globals_'. 

        Debugger.core.start settings are passed via optional
        dictionary `start_opts'. Overall debugger settings are in
        Debugger.settings which changed after an instance is created
        . Also see `run_eval' if what you want to run is an
        run_eval'able expression have that result returned and
        `run_call' if you want to debug function run_call.
        """
        if globals_ is None:
            globals_ = globals()
        if locals_ is None:
            locals_ = globals_
        if not isinstance(cmd, types.CodeType):
            cmd = cmd+'\n'
            pass
        self.core.start(start_opts)
        try:
            exec(cmd, globals_, locals_)
        except Mexcept.DebuggerQuit:
            pass
        finally:
            self.core.stop()
        return
    def run_call(self, func, start_opts=None, *args, **kwds):
        """ Run debugger on function call: `func(*args, **kwds)'

        See also `run_eval' if what you want to run is an eval'able
        expression have that result returned and `run' if you want to
        debug a statment via exec.
        """
        res = None
        self.core.start(opts=start_opts)
        try:
            res = func(*args, **kwds)
        except Mexcept.DebuggerQuit:
            pass
        finally:
            self.core.stop()
        return res

    def run_eval(self, expr, start_opts=None, globals_=None, locals_=None):
        """ Run debugger on string `expr' which will executed via the
        built-in Python function: eval; `globals_' and `locals_' are
        the dictionaries to use for local and global variables. If
        `globals' is not given, __main__.__dict__ (the current global
        variables) is used. If `locals_' is not given, it becomes a
        copy of `globals_'.

        See also `run_call' if what you to debug a function call and
        `run' if you want to debug general Python statements.
        """
        if globals_ is None:
            globals_ = globals()
        if locals_ is None:
            locals_ = globals_
        if not isinstance(expr, types.CodeType):
            self.eval_string = expr
            expr = expr+'\n'
            pass
        retval = None
        self.core.start(start_opts)
        try:
            retval = eval(expr, globals_, locals_)
        except Mexcept.DebuggerQuit:
            pass
        finally:
            self.core.stop()
        return retval

    def run_script(self, filename, start_opts=None, globals_=None, 
                   locals_=None):
        """ Run debugger on Python script `filename'. The script may
        inspect sys.argv for command arguments. `globals_' and
        `locals_' are the dictionaries to use for local and global
        variables. If `globals' is not given, globals() (the current
        global variables) is used. If `locals_' is not given, it
        becomes a copy of `globals_'.

        True is returned if the program terminated normally and False
        if the debugger initiated a quit or the program did not normally
        terminate.

        See also `run_call' if what you to debug a function call,
        `run_eval' if you want to debug an expression, and `run' if you
        want to debug general Python statements not inside a file.
        """
        self.mainpyfile = self.core.canonic(filename)

        # Start with fresh empty copy of globals and locals and tell the script
        # that it's being run as __main__ to avoid scripts being able to access
        # the pydb.py namespace.
        if globals_ is None:
            import __main__
            globals_ = {"__name__" : "__main__",
                       "__file__" : self.mainpyfile,
                       "__builtins__" : __builtins__
                       }
        if locals_ is None:
            locals_ = globals_
        self.core.start(start_opts)
        retval = False
        self.core.execution_status = 'Running'
        try:
            exec(compile(open(self.mainpyfile).read(), self.mainpyfile, 'exec'), globals_, locals_)
            retval = True
        except  SyntaxError:
            print(sys.exc_info()[1])
            retval = False
            pass
        except IOError:
            print(sys.exc_info()[1])
        except Mexcept.DebuggerQuit:
            retval = False
            pass
        except Mexcept.DebuggerRestart:
            self.core.execution_status = 'Restart requested'
            raise Mexcept.DebuggerRestart
        finally:
            self.core.stop(options={'remove': True})
        return retval

    def restart_argv(self):
        '''Return an array that would be execv-ed  to restart the program'''
        return self.orig_sys_argv or self.program_sys_argv

    # Note: has to come after functions listed in ignore_filter.
    DEFAULT_INIT_OPTS = {
        # What routines will we not trace into? 
        'ignore_filter': tracefilter.TraceFilter(
            [tracer.start, tracer.stop, 
             run_eval, run_call, run_eval, run_script]),

        # sys.argv when not None contains sys.argv *before* debugger
        # command processing. So sys.argv contains debugger options as
        # well as debugged-program options. These options are used to
        # do a "hard" or execv() restart.

        # program_sys_argv is set by option save_sys_argv and contains
        # sys.argv that we see now which may have debugger options
        # stripped, or it may be that we were not called from a
        # debugger front end but from inside the running
        # program. These options are suitable for a "soft" or
        # exception-handling DebuggerRestart kind of restart.
        'orig_sys_argv' : None,
        'save_sys_argv' : True,

        # How is I/O for this debugger handled?
        'activate'    : False, 
        'interface'   : Muser.UserInterface(),
        'input'       : None,
        'output'      : None,
        'processor'   : None,
        
        # Setting contains lots of debugger settings - a whole file
        # full of them!
        'settings'    : Mdefault.DEBUGGER_SETTINGS,

        'start_opts'  : Mdefault.START_OPTS,
        'step_ignore' : 0,
        }

    def __init__(self, opts=None):
        """Create a debugger object. But depending on the value of
        key 'start' inside hash 'opts', we may or may not initially
        start debugging.

        See also Debugger.start and Debugger.stop.
        """

        self.mainpyfile  = None
        self.thread      = None
        self.eval_string = None
        get_option = lambda key: Mmisc.option_set(opts, key, 
                                                  self.DEFAULT_INIT_OPTS)

        # set the instance variables that come directly from options.
        for opt in ['settings', 'orig_sys_argv']:
            setattr(self, opt, get_option(opt))
            pass

        core_opts = {}
        for opt in ('ignore_filter', 'proc_opts', 'processor', 'step_ignore', 
                    'processor',):
            core_opts[opt] = get_option(opt)
            pass

        # How is I/O for this debugger handled? This should
        # be set before calling DebuggerCore.
        self.intf = [get_option('interface')]

        inp = get_option('input')
        if inp:
            self.intf[-1].input = inp
            pass

        out = get_option('output')
        if out:
            self.intf[-1].output = out
            pass

        self.core = Mcore.DebuggerCore(self, core_opts)
        self.core.add_ignore(self.core.stop)

        # When set True, we'll also suspend our debug-hook tracing.
        # This gives us a way to prevent or allow self debugging.
        self.core.trace_hook_suspend = False

        if get_option('save_sys_argv'):
            # Save the debugged program's sys.argv? We do this so that
            # when the debugged script munges these, we have a good
            # copy to use for an exec restart
            self.program_sys_argv = list(sys.argv)
        else:
            self.program_sys_argv = None
            pass

        self.sigmgr = Msig.SignalManager(self)

        # Were we requested to activate immediately? 
        if get_option('activate'): 
            self.core.start(get_option('start_opts'))
            pass
        return
    pass

# Demo it
if __name__=='__main__':
    def foo():
        y = 2
        for i in range(2):
            print("%d %d" % (i, y) )
            pass
        return 3
    import debugger
    d = debugger.Debugger()
    d.settings['trace'] = True
    d.settings['printset'] = tracer.ALL_EVENTS
    d.core.step_ignore = -1
    print('Issuing: run_eval("1+2")')
    print(d.run_eval('1+2'))
    print('Issuing: run_exec("x=1; y=2")')
    d.run_exec('x=1; y=2')

    print('Issuing: run("3*4")')
    print(d.run('3*4'))
    print('Issuing: run("x=3; y=4")')
    d.run('x=3; y=4')

    print('Issuing: run_call(foo)')
    d.run_call(foo)
    if len(sys.argv) > 1:
        while True:
            try:
                print('started')
                d.core.step_ignore = 0
                d.core.start()
                x = 4
                x = foo()
                for i in range(2):
                    print("%d" % (i+1)*10)
                    pass
                d.core.stop()
                def square(x): return x*x
                print('calling: run_call(square,2)') 
                d.run_call(square, 2)
            except Mexcept.DebuggerQuit:
                print("That's all Folks!...")
                break
            except Mexcept.DebuggerRestart:
                print('Restarting...')
                pass
            pass
        pass
    pass
 



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from import_relative import import_relative

Mbase_cmd  = import_relative('base_cmd', top_name='pydbgr')
Mcmdfns    = import_relative('cmdfns', '..', 'pydbgr')
Mfile      = import_relative('file', '...lib', 'pydbgr')
Mmisc      = import_relative('misc', '...', 'pydbgr')
Mbreak     = import_relative('break', '.', 'pydbgr')

class DeleteCommand(Mbase_cmd.DebuggerCommand):
    """**delete** [*bpnumber* [*bpnumber*...]]

Delete some breakpoints.

Arguments are breakpoint numbers with spaces in between.  To delete
all breakpoints, give no argument.  those breakpoints.  Without
argument, clear all breaks (but first ask confirmation).
    
See also the `clear` command which clears breakpoints by line/file
number."""

    category      = 'breakpoints'
    min_args      = 0
    max_args      = None
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = False
    short_help    = 'Delete some breakpoints or auto-display expressions'

    def run(self, args):
        if len(args) <= 1:
            if self.confirm('Delete all breakpoints', False):
                self.core.bpmgr.delete_all_breakpoints()
            return

        for arg in args[1:]:
            i = self.proc.get_int(arg, min_value=1, default=None,
                                  cmdname='delete')
            if i is None: continue

            success, msg = self.core.bpmgr.delete_breakpoint_by_number(i)
            if not success:
                self.errmsg(msg)
            else:
                self.msg('Deleted breakpoint %d' % i)
                pass
            pass
        return
        

if __name__ == '__main__':
    Mdebugger = import_relative('debugger', '...')
    d = Mdebugger.Debugger()
    command = DeleteCommand(d.core.processor)
    pass



#  Copyright (C) 2013 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from import_relative import import_relative
# Our local modules
Mbase_cmd = import_relative('base_cmd', top_name='pydbgr')

class AliasCommand(Mbase_cmd.DebuggerCommand):
    """**alias** *alias-name* *debugger-command*

Add alias *alias-name* for a debugger command *debugger-comand*.

Add an alias when you want to use a command abbreviation for a command
that would otherwise be ambigous. For example, by default we make `s`
be an alias of `step` to force it to be used. Without the alias, `s`
might be `step`, `show`, or `set` among others

**Example:**

    alias cat list   # "cat prog.py" is the same as "list prog.py"
    alias s   step   # "s" is now an alias for "step".
                     # The above example is done by default.

See also `unalias` and `show alias`."""
    
    category      = 'support'
    min_args      = 0
    max_args      = 2  # Need at most this many
    name          = 'alias'
    need_stack    = True
    short_help    = 'Add an alias for a debugger command'
  
    # Run command. 
    def run(self, args):
        if len(args) == 1:
            self.proc.commands['show'].run(['show', 'alias'])
        elif len(args) == 2:
            self.proc.commands['show'].run(['show', 'alias', args[1]])
        else:
            junk, al, command = args
            if command in self.proc.commands:
                if al in self.proc.aliases:
                    old_command = self.proc.aliases[al]
                    self.msg(("Alias '%s#' for command '%s'replaced old " + 
                              "alias for '%s'.") %
                             (al, command, od_command, old_command))
                else:
                    self.msg("New alias '%s' for command '%s' created." % 
                             (al, command))
                    pass
                self.proc.aliases[al] = command
            else:
                self.errmsg(("You must alias to a command name, and '%s' " +
                             'and is not one.') % command)  
                pass
            return
        pass
    pass

if __name__ == '__main__':
    # Demo it.
    mock = import_relative('mock')
    d, cp = mock.dbg_setup()
    command      = AliasCommand(cp)
    command.run(['alias', 'yy', 'foo'])
    command.run(['alias', 'yy',' foo'])
    command.run(['alias', 'yy' 'step'])
    command.run(['alias'])
    command.run(['alias', 'yy' 'next'])
    pass



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import atexit, os
from import_relative import import_relative

# Our local modules
Mdebugger  = import_relative('debugger', '...', 'pydbgr')
Mbase_cmd  = import_relative('base_cmd', top_name='pydbgr')
Mcomcodes  = import_relative('comcodes', '...interfaces', 'pydbgr')
debugger   = import_relative('debugger', '...')
Mmisc      = import_relative('misc', '...', 'pydbgr')

class RestartCommand(Mbase_cmd.DebuggerCommand):
    """**restart**

Restart debugger and program via an *exec()* call. All state is lost,
and new copy of the debugger is used."""

    category      = 'support'
    min_args      = 0
    max_args      = 0
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = False
    short_help    = '(Hard) restart of program via execv()'

    def run(self, args):
        sys_argv = self.debugger.restart_argv()
        if sys_argv and len(sys_argv) > 0:
            confirmed = self.confirm('Restart (execv)', False)
            if confirmed: 
                self.msg(Mmisc.wrapped_lines("Re exec'ing:", repr(sys_argv),
                                             self.settings['width']))
                # Run atexit finalize routines. This seems to be Kosher:
                # http://mail.python.org/pipermail/python-dev/2009-February/085791.html
                try:
                    atexit._run_exitfuncs()
                except:
                    pass
                os.execvp(sys_argv[0], sys_argv)
                pass
            pass
        else:
            self.errmsg("No executable file and command options recorded.")
            pass
        return
    pass

if __name__ == '__main__':
    mock = import_relative('mock')
    d, cp = mock.dbg_setup()
    command = RestartCommand(cp)
    command.run([])
    import sys
    if len(sys.argv) > 1:
        # Strip of arguments so we don't loop in exec.
        d.orig_sys_argv = ['python', sys.argv[0]] 
        command.run([])
        pass
    pass





#!/usr/bin/env python
'Unit test for pydbgr.processor.cmdproc'
import os, unittest
from import_relative import import_relative

Mcore = import_relative('lib.core', '...pydbgr')

class MockProcessor:
    pass

class TestCore(unittest.TestCase):

    def test_is_next_stop(self):
        # We assume there's at least one command
        opts = {'processor': MockProcessor()}
        dc = Mcore.DebuggerCore(None, opts=opts)
        dc.step_ignore = 1
        self.assertFalse(dc._is_step_next_stop('line'))
        self.assertEqual(0, dc.step_ignore)
        self.assertTrue(dc._is_step_next_stop('line'))
        self.assertEqual(0, dc.step_ignore)

        return

    def test_canonic(self):
        opts = {'processor': MockProcessor()}
        dc = Mcore.DebuggerCore(None, opts=opts)
        s = '<string>'
        self.assertEqual(s, dc.canonic(s), 
                         'canonic should not have changed string')
        self.assertEqual(os.path.sep, dc.canonic(__file__)[0],
                         'canonic should produce an absolute file')
        return

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2009-2010, 2013 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from import_relative import import_relative
# Our local modules

Mbase_subcmd = import_relative('base_subcmd', '..', 'pydbgr')
Mcmdfns      = import_relative('cmdfns', '...', 'pydbgr')

class SetMaxString(Mbase_subcmd.DebuggerSubcommand):
    "Set maximum length to show string output"
    
    in_list    = True
    min_abbrev = len('str') # Need at least "set max str"

    def run(self, args):
        Mcmdfns.run_set_int(self, ' '.join(args),
                            "The '%s' command requires a character count" % self.name,
                            0, None)
        self.proc._repr.maxstring =  self.settings[self.name]
        return None
    pass





# -*- coding: utf-8 -*-
#   Copyright (C) 2009-2010, 2013 Rocky Bernstein rocky@gnu.org
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from import_relative import import_relative
Mbase_subcmd  = import_relative('base_subcmd', '..', 'pydbgr')

class SetTrace(Mbase_subcmd.DebuggerSetBoolSubcommand):

    """Set event tracing.

See also "set events","set trace", and "show trace".
"""

    in_list    = True
    min_abbrev = len('trace')  # Must use at least "set trace"
    short_help = "Set execution tracing"
    pass

if __name__ == '__main__':
    Mhelper = import_relative('__demo_helper__', '.', 'pydbgr')
    sub = Mhelper.demo_run(SetTrace)
    d = sub.proc.debugger
    for args in (['on'], ['off']):
        sub.run(args)
        print(d.settings['trace'])
        pass
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013 Rocky Bernstein
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
#    02110-1301 USA.

import sys, time
# Our local modules
from import_relative import import_relative
import trepan.interface
Mmisc = import_relative('misc', '.', 'trepan')
Mclient   = import_relative('client', '.interfaces', 'trepan')
Mcomcodes = import_relative('comcodes', '.interfaces', 'trepan')

#
# Connects to a debugger in server mode
#

DEFAULT_CLIENT_CONNECTION_OPTS = {'open': True, 'IO': 'TCP'}
def start_client(connection_opts):
      intf = Mclient.ClientInterface(connection_opts=connection_opts)
      # debugger.interface.append(intf)
      intf.msg("Connected.")
      done=False
      while not done:
            control, remote_msg = intf.read_remote()
            # print 'c, r', control, remote_msg
            if Mcomcodes.PRINT == control:
                  print(remote_msg,)
                  pass
            elif control in [Mcomcodes.CONFIRM_TRUE, Mcomcodes.CONFIRM_FALSE]:
                  default = (Mcomcodes.CONFIRM_TRUE == control)
                  if intf.confirm(remote_msg.rstrip('\n'), default):
                        msg='Y'
                  else:
                        msg='N'
                        pass
                  intf.write_remote(Mcomcodes.CONFIRM_REPLY, msg)
            elif Mcomcodes.PROMPT == control:
                  msg = intf.read_command('(Trepan*) ').strip()
                  intf.write_remote(Mcomcodes.CONFIRM_REPLY, msg)
            elif Mcomcodes.QUIT == control:
                  print('Quitting...')
                  done = True
                  break
            elif Mcomcodes.RESTART == control:
                  # FIXME need to save stuff like port # and
                  # and for FIFO we need new pid.
                  if 'TCP' == connection_opts['IO']:
                        print('Restarting...')
                        intf.inout.close()
                        time.sleep(1)
                        intf.inout.open()
                  else:
                        print("Don't know how to hard-restart FIFO...")
                        done=True
                        pass
                  break
            else:
                  print("!! Weird status code received '%s'" % control)
                  print(remote_msg,)
                  pass
            pass
      intf.close()
      pass

def main():
      if len(sys.argv) > 1:
            remote_opts = {'open': int(sys.argv[1]), 'IO': 'FIFO'}
      else:
            remote_opts = {'open': True, 'IO': 'TCP'}
            pass
      start_client(remote_opts)
      return

if __name__ == '__main__':
      main()
      pass



import threading, time

class BgThread(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        return

    def run(self):
        print "Sleeping for 1"
        time.sleep(1)
        print 'Finished sleep'
        return

background = BgThread()
background.start()
print 'The main program continues to run in foreground.'

background.join()    # Wait for the background task to finish
print 'Main program waited until background was done.'



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2009, 2010 Rocky Bernstein
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inspect, os, pyficache

from import_relative import import_relative
Mmisc = import_relative('misc', '..')

def set_break(cmd_obj, func, filename, lineno, condition, temporary, args):
    if lineno is None:
        part1 = ("I don't understand '%s' as a line number, function name,"
                 % ' '.join(args[1:]))
        msg = Mmisc.wrapped_lines(part1, "or file/module plus line number.",
                                  cmd_obj.settings['width'])
        cmd_obj.errmsg(msg)
        return False
    if filename is None:
        filename = cmd_obj.proc.curframe.f_code.co_filename
        filename = cmd_obj.core.canonic(filename)
        pass
    if func is None:
        ok_linenos = pyficache.trace_line_numbers(filename)
        if not ok_linenos or lineno not in ok_linenos:
            part1 = ('File %s' % cmd_obj.core.filename(filename))
            msg = Mmisc.wrapped_lines(part1, 
                                      "is not stoppable at line %d." %
                                      lineno, cmd_obj.settings['width'])
            cmd_obj.errmsg(msg)
            return False
        pass
    bp =  cmd_obj.core.bpmgr.add_breakpoint(filename, lineno, temporary, 
                                         condition, func)
    if func:
        cmd_obj.msg('Breakpoint %d set on calling function %s()' 
                 % (bp.number, func.func_name))
        part1 = 'Currently this is line %d of file'  % lineno
        msg = Mmisc.wrapped_lines(part1, cmd_obj.core.filename(filename),
                                  cmd_obj.settings['width'])
    else:
        part1 = ( 'Breakpoint %d set at line %d of file' 
                  % (bp.number, lineno))
        msg = Mmisc.wrapped_lines(part1, cmd_obj.core.filename(filename),
                                  cmd_obj.settings['width'])
        pass
    cmd_obj.msg(msg)
    return True

def parse_break_cmd(cmd_obj, args):
    curframe = cmd_obj.proc.curframe
    if 0 == len(args) or args[0] == 'if':
        filename = cmd_obj.core.canonic(curframe.f_code.co_filename)
        lineno   = curframe.f_lineno
        if 0 == len(args):
            return (None, filename, lineno, None)
        modfunc = None
        condition_pos = 0
    else:
        (modfunc, filename, lineno) = cmd_obj.proc.parse_position(args[0])
        condition_pos = 1
        pass
    if inspect.ismodule(modfunc) and lineno is None and len(args) > 1:
        val = cmd_obj.proc.get_an_int(args[1], 
                                   'Line number expected, got %s.' % 
                                   args[1])
        if val is None: return (None, None, None, None)
        lineno = val
        condition_pos = 2
        pass
    if len(args) > condition_pos and 'if' == args[condition_pos]:
        condition = ' '.join(args[condition_pos+1:])
    else:
        condition = None
        pass
    if inspect.isfunction(modfunc):
        func = modfunc
    else:
        func = None
    return (func, filename, lineno, condition)




#!/usr/bin/env python
'Unit test for pydbgr.processor.cmdproc'
import inspect, os, sys, types, unittest
from import_relative import import_relative

Mcmdproc = import_relative('processor.cmdproc', '...pydbgr')
Mmock    = import_relative('processor.command.mock', '...pydbgr')

class TestCmdProc(unittest.TestCase):

    def setUp(self):
        self.errors             = []
        self.msgs               = []
        self.d                  = Mmock.MockDebugger()
        self.cp                 = Mcmdproc.CommandProcessor(self.d.core)
        self.cp.intf[-1].msg    = self.msg 
        self.cp.intf[-1].errmsg = self.errmsg
        return

    def errmsg(self, msg):
        self.errors.append(msg)
        return

    def msg(self, msg):
        self.msg.append(msg)
        return

    def test_basic(self):

        # We assume there's at least one command
        self.assertTrue(len(self.cp.commands) > 0)
        self.assertTrue(len(self.cp.aliases) > 0)

        # In fact we assume there is a 'quit' command...
        self.assertEqual('quit', Mcmdproc.resolve_name(self.cp, 'quit'))
        #   with alias 'q'
        self.assertEqual('quit', Mcmdproc.resolve_name(self.cp, 'q'))
#         processor.cmdproc.print_source_line(self.msg, 100, 
#                                             'source_line_test.py')

        self.cp.frame = sys._getframe()
        self.cp.setup()
        self.assertEqual(3, self.cp.eval('1+2'))
        return

    def test_class_vars(self):
        '''See that each command has required attributes defined.  Possibly in
        a strongly-typed language we would not need to do much of this.'''

        for cmd in list(self.cp.commands.values()):

            name = cmd.__class__
            for attr in ['aliases', 'min_args', 'max_args', 'name', 
                         'need_stack']:
                self.assertTrue(hasattr(cmd, attr),
                                '%s command should have a %s attribute' %
                                (name, attr))
                pass

            for attr in ['category', 'short_help']:
                self.assertTrue(hasattr(cmd, attr),
                                '%s command should have a %s attribute' %
                                (name, attr))
                value = getattr(cmd, attr)
                self.assertEqual(types.StringType, type(value),
                                '%s command %s attribute should be a string' %
                                (name, attr))
                pass

            self.assertEqual(types.StringType, type(cmd.name))
            self.assertEqual(types.TupleType, type(cmd.aliases),
                             '%s aliases should be a tuple type' % 
                             repr(cmd.aliases))
            for value in cmd.aliases:
                self.assertEqual(types.StringType, type(value),
                                '%s command aliases should be strings' %
                                 name)

            if cmd.min_args is not None:
                if cmd.max_args is not None:
                    self.assertTrue(cmd.min_args <= cmd.max_args,
                                    "%s min_args: %d, max_args: %d" % 
                                    (name, cmd.min_args, cmd.max_args))
                    pass
                pass
            pass

    def test_args_split(self):
        for test, expect in (
            ("Now is the time",    [['Now', 'is', 'the', 'time']],),
            ("Now is the time ;;", [['Now', 'is', 'the', 'time'], []],),
            ("Now is 'the time'",  [['Now', 'is', "'the time'"]]),
            ("Now is the time ;; for all good men",
             [['Now', 'is', 'the', 'time'], ['for', 'all', 'good', 'men']],),
            ("Now is the time ';;' for all good men",
             [['Now', 'is', 'the', 'time', "';;'", 
               'for', 'all', 'good', 'men']],)
            ):
            self.assertEqual(expect, Mcmdproc.arg_split(test))
            pass
        return

    def test_parse_position(self):
        self.cp.frame = sys._getframe()
        self.cp.setup()

        # See that we can parse a file/line combo: e.g. filename.py:10 
        filename = os.path.realpath(os.path.abspath(__file__))
        modfunc, f, l = self.cp.parse_position("%s:10" % filename)
        self.assertEqual(filename, f, 'file:line parsing bolixed')

        # Without the line number should be a problem though
        modfunc, f, l = self.cp.parse_position(filename)
        self.assertEqual(None, f, 'file should not work')
        return 


    def test_parse_position_one_arg(self):
        self.assertEqual((None, None, None), 
                         self.cp.parse_position_one_arg('4+1'))
        self.cp.frame = sys._getframe()
        self.cp.setup()
        modfunc, f, l = self.cp.parse_position_one_arg('4+1')
        self.assertEqual(5, l)
        self.assertTrue(f.endswith('test-cmdproc.py'))
        self.assertEqual(None, modfunc)

        # See that we can a module name
        modfunc, f, l = self.cp.parse_position_one_arg('os.path')
        self.assertTrue(inspect.ismodule(modfunc),
                        'Module name, e.g. os.path bolixed')

        def foo(): pass
        for name in ('os.path.join', 'foo', 'self.test_parse_position_one_arg'):
            modfunc, f, l = self.cp.parse_position_one_arg(name)
            self.assertTrue(inspect.isfunction(modfunc),
                            'function name %s bolixed' % name)
            pass
        return

    def test_preloop_hooks(self):
        fn = self.cp.commands['list']
        self.assertEqual(0, len(self.cp.preloop_hooks),
                         'Should start out with no preloop hooks')
        self.assertEqual(False, self.cp.remove_preloop_hook(fn),
                         'Should not be able to return a non-existent hook')
        self.cp.add_preloop_hook(fn)
        self.assertEqual(1, len(self.cp.preloop_hooks),
                         'Should now have one preloop hook added')
        self.assertEqual(True, self.cp.remove_preloop_hook(fn))
        self.assertEqual(0, len(self.cp.preloop_hooks),
                         'Should be back to no preloop hooks')
        # FIXME try adding and running a couple of hooks.
        return

if __name__ == '__main__':
    unittest.main()
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Subsidiary routines used to "pack" and "unpack" TCP messages. """

TCP_MAX_PACKET = 8192 # Largest size for a recv
LOG_MAX_MSG    = 4    # int(log(TCP_MAX_PACKET)
def pack_msg(msg):
    fmt = '%%0%dd' % LOG_MAX_MSG # A funny way of writing: '%04d'
    return ( fmt % len(msg)) + msg

def unpack_msg(buf):
    length  = int(buf[0:LOG_MAX_MSG])
    data    = buf[LOG_MAX_MSG:LOG_MAX_MSG+length]
    buf     = buf[LOG_MAX_MSG+length:]
    return buf, data

# Demo
if __name__=='__main__':
    msg = "Hi there!"
    assert unpack_msg(pack_msg(msg))[1] == msg
    pass




#!/usr/bin/env python
'Unit test for pydbgr.processor.command.step'
import sys, unittest

from import_relative import import_relative
Mfinish = import_relative('processor.command.finish', '...pydbgr')

from cmdhelper import dbg_setup

class TestFinishCommand(unittest.TestCase):
    """Tests FinishCommand class"""

    def test_finish(self):
        """Test processor.command.finish.FinishCommand.run()"""
        d, cp = dbg_setup()
        command = Mfinish.FinishCommand(cp)
        for c in ((['finish', '5'], True,),
                  (['finish', '0*5+1'], True)):
            command.continue_running = False
            command.proc.stack = [(sys._getframe(0), 14,)]
            result = command.run(c[0])
            self.assertEqual(c[1], result)
            pass
        return

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#  Copyright (C) 2012-2013 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from import_relative import import_relative
Mbase_cmd  = import_relative('base_cmd', top_name='pydbgr')
Mprint     = import_relative('print', '...lib', 'pydbgr')
Meval      = import_relative('eval',  '...lib', 'pydbgr')

class EvalCommand(Mbase_cmd.DebuggerCommand):
    """**eval** *python-statement*

Run *python-statement* in the context of the current frame.

If no string is given, we run the string from the current source code
about to be run. If the command ends `?` (via an alias) and no string is
given, the following translations occur:

   {if|elif} <expr> :  => <expr>
   while <expr> :      => <expr>
   return <expr>       => <expr>
   <var> = <expr>      => <expr>

The above is done via regular expression matching. No fancy parsing is
done, say, to look to see if *expr* is split across a line or whether
var an assignment might have multiple variables on the left-hand side.

**Examples:**

    eval 1+2  # 3
    eval      # Run current source-code line
    eval?     # but strips off leading 'if', 'while', ..
              # from command

See also `set autoeval`, `pr`, `pp` and `examine`.
"""
    aliases       = ('eval?','?')
    category      = 'data'
    min_args      = 0
    max_args      = None
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = True
    short_help    = 'Print value of expression EXP'

    def run(self, args):
        if 1 == len(args):
            text = self.proc.current_source_text.rstrip('\n')
            if '?' == args[0][-1]:
                text = Meval.extract_expression(text)
                self.msg("eval: %s" % text)
                pass
            
        else:
            text = self.proc.current_command[len(self.proc.cmd_name):]
            pass
        text = text.strip()
        try:
            self.proc.exec_line(text)
        except:
            pass

if __name__ == '__main__':
    import inspect
    cmdproc     = import_relative('cmdproc', '..')
    debugger    = import_relative('debugger', '...')
    d           = debugger.Debugger()
    cp          = d.core.processor
    cp.curframe = inspect.currentframe()
    command = EvalCommand(cp)
    me = 10
    
    # command.run([command.name, '1+2'])
    # command.run([command.name, 'if 5: x=1'])
    pass





# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Module for client (i.e. user to communication-device) interaction.
The debugged program is at the other end of the communcation."""
import atexit

# Our local modules
from import_relative import *
Muser       = import_relative('user', top_name='pydbgr')
Mtcpclient  = import_relative('tcpclient', '..io', 'pydbgr')
Mfifoclient = import_relative('fifoclient', '..io', 'pydbgr')
Mcomcodes   = import_relative('comcodes', '.', 'pydbgr')
Mmisc       = import_relative('misc', '..', 'pydbgr')

class ClientInterface(Muser.UserInterface):
    """Interface for a user which is attached to a debugged process
    via some sort of communication medium (e.g. socket, tty, FIFOs).
    This could be on the same computer in a different process or on
    a remote computer."""

    DEFAULT_INIT_CONNECTION_OPTS = {'IO': 'FIFO'}
    def __init__(self, inp=None, out=None, inout=None, user_opts=None, 
                 connection_opts=None):
        get_connection_option = lambda key: \
            Mmisc.option_set(connection_opts, key, 
                             self.DEFAULT_INIT_CONNECTION_OPTS)
        Muser.UserInterface.__init__(self, inp, out, user_opts)
        
        self.inout = None # initialize in case assignment below fails
        if inout:
            self.inout = inout 
        else:
            self.server_type = get_connection_option('IO')
            if 'FIFO' == self.server_type:
                self.inout = Mfifoclient.FIFOClient(opts=connection_opts)
            elif 'TCP' == self.server_type:
                self.inout = Mtcpclient.TCPClient(opts=connection_opts)
            else:
                self.errmsg("Expecting server type TCP or FIFO. Got: %s." %
                            self.server_type)
                return
            pass
        return

    def read_remote(self):
        '''Send a message back to the server (in contrast to
        the local user output channel).'''
        coded_line = self.inout.read_msg()
        control = coded_line[0]
        remote_line = coded_line[1:]
        return (control, remote_line)

    def write_remote(self, code, msg):
        '''Send a message back to the server (in contrast to
        the local user output channel).'''
        # FIXME change into write_xxx
        return self.inout.writeline(code + msg)
    pass
    
# Demo
if __name__=='__main__':
    intf = ClientInterface()



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2012-2013 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import inspect, os, linecache, pyficache, sys

# Our local modules
from import_relative import import_relative

Mbase_cmd = import_relative('base_cmd', top_name='pydbgr')
Mcmdfns   = import_relative('cmdfns', '..', 'pydbgr')
Mfile     = import_relative('file', '...lib', 'pydbgr')

def pyc2py(filename):
    if '.pyc' == filename[-4:]:
        return filename[:-1]
    return filename

class ListCommand(Mbase_cmd.DebuggerCommand):
    """**list** [*module] [*first* [*num*]]
list *location* [*num*]

List source code.

Without arguments, print lines centered around the current line. If
*num* is given that number of lines is shown.

If this is the first `list` command issued since the debugger command
loop was entered, then the current line is the current frame. If a
subsequent list command was issued with no intervening frame changing,
then that is start the line after we last one previously shown.

A *location* is either:

  - a number, e.g. 5,
  - a function, e.g. join or os.path.join
  - a module, e.g. os or os.path
  - a filename, colon, and a number, e.g. foo.py:5,
  - or a module name and a number, e.g,. os.path:5.
  - a '.' for the current line number
  - a '-' for the lines before the current linenumber

If the location form is used with a subsequent parameter, the
parameter is the starting line number is used. When there two numbers
are given, the last number value is treated as a stopping line unless
it is less than the start line, in which case it is taken to mean the
number of lines to list instead.

Wherever a number is expected, it does not need to be a constant --
just something that evaluates to a positive integer.

**Examples:**

    list 5            # List starting from line 5
    list 4+1          # Same as above.
    list foo.py:5     # List starting from line 5 of foo.py
    list os.path:5    # List starting from line 5 of os.path
    list os.path 5    # Same as above.
    list os.path 5 6  # list lines 5 and 6 of os.path
    list os.path 5 2  # Same as above, since 2 < 5.
    list foo.py:5 2   # List two lines starting from line 5 of foo.py
    list os.path.join # List lines around the os.join.path function.
    list .            # List lines centered from where we currently are stopped
    list -            # List lines previous to those just shown

See `set listize` or `show listsize` to see or set the value.
"""

    aliases       = ('l',)
    category      = 'files'
    min_args      = 0
    max_args      = 3
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = False
    short_help    = 'List source code'

    # What a f*cking mess. Necessitated I suppose because we want to
    # allow somewhat flexible parsing with either module names, files or none
    # and optional line counts or end-line numbers.
    def parse_list_cmd(self, args):
        """Parses arguments for the "list" command and returns the tuple:
        filename, start, last
        or sets these to None if there was some problem."""

        dbg_obj = self.core.debugger
        curframe = self.proc.curframe

        if curframe:
            filename = curframe.f_code.co_filename
        else:
            filename = None
            pass

        last = None
        listsize = dbg_obj.settings['listsize']
        if len(args) == 0 and not curframe:
            self.errmsg("No Python program loaded.")
            return (None, None, None)

        if len(args) > 0:
            if args[0] == '-':
                first = max(1, self.proc.list_lineno - 2*listsize - 1)
            elif args[0] == '.':
                first = max(1, inspect.getlineno(curframe) - int(listsize/2))
            else:
                (modfunc, filename, first) = self.proc.parse_position(args[0])
                if first == None and modfunc == None:
                    # error should have been shown previously
                    return (None, None, None)
                if len(args) == 1:
                    if first is None and modfunc is not None: first = 1
                    first = max(1, first - int(listsize/2))
                elif len(args) == 2 or (len(args) == 3 and modfunc):
                    msg = 'Starting line expected, got %s.' % args[-1]
                    num = self.proc.get_an_int(args[1], msg)
                    if num is None: return (None, None, None)
                    if modfunc:
                        if first is None:
                            first = num
                            if len(args) == 3 and modfunc:
                                msg = ('last or count parameter expected, ' +
                                       'got: %s.' % args[2])
                                last = self.proc.get_an_int(args[2], msg)
                                pass
                            pass
                        else:
                            last = num
                            pass
                    else:
                        last = num
                        pass
                    if last is not None and last < first:
                        # Assume last is a count rather than an end line number
                        last = first + last - 1
                        pass
                    pass
                elif not modfunc:
                    self.errmsg(('At most 2 parameters allowed when no module' +
                                 ' name is found/given. Saw: %d parameters')
                                % len(args))
                    return (None, None, None)
                else:
                    self.errmsg(('At most 3 parameters allowed when a module' +
                                 ' name is given. Saw: %d parameters') %
                                len(args))
                    return (None, None, None)
                pass
        elif self.proc.list_lineno is None and self.core.is_running():
            first = max(1, inspect.getlineno(curframe) - int(listsize/2))
        else:
            first = self.proc.list_lineno + 1
            pass
        if last is None:
            last = first + listsize - 1
            pass

        return (filename, first, last)

    def run(self, args):
        filename, first, last = self.parse_list_cmd(args[1:])
        curframe = self.proc.curframe
        if filename is None: return
        filename = pyc2py(filename)

        # We now have range information. Do the listing.
        max_line = pyficache.size(filename)
        if max_line is None:
            self.errmsg('No file %s found' % filename)
            return

        canonic_filename = os.path.realpath(os.path.normcase(filename))
        if first > max_line:
            self.errmsg('Bad start line %d - file "%s" has only %d lines'
                        % (first, filename, max_line))
            return

        if last > max_line:
            self.msg('End position changed to last line %d ' % max_line)
            last = max_line

        bplist = self.core.bpmgr.bplist
        opts = {
            'reload_on_change' : self.settings['reload'],
            'output'           : self.settings['highlight'],
            'strip_nl'         : False
            }

        try:
            for lineno in range(first, last+1):
                line = pyficache.getline(filename, lineno, opts)
                if line is None:
                    line = linecache.getline(filename, lineno, self.proc.frame.f_globals)
                    pass
                if line is None:
                    self.msg('[EOF]')
                    break
                else:
                    line = line.rstrip('\n')
                    s = self.proc._saferepr(lineno).rjust(3)
                    if len(s) < 5: s += ' '
                    if (canonic_filename, lineno,) in list(bplist.keys()):
                        bp    = bplist[(canonic_filename, lineno,)][0]
                        a_pad = '%02d' % bp.number
                        s    += bp.icon_char()
                    else:
                        s    += ' '
                        a_pad = '  '
                        pass
                    if curframe and lineno == inspect.getlineno(curframe) \
                       and filename == curframe.f_code.co_filename:
                        s += '->'
                    else:
                        s += a_pad
                        pass
                    self.msg(s + '\t' + line)
                    self.proc.list_lineno = lineno
                    pass
                pass
        except KeyboardInterrupt:
            pass
        return False

if __name__ == '__main__':
    from mock import MockDebugger
    d = MockDebugger()
    command = ListCommand(d.core.processor)
    command.run(['list'])
    cmdproc = import_relative('cmdproc', '..')
    command.proc = d.core.processor = cmdproc.CommandProcessor(d.core)
    command = ListCommand(d.core.processor)
    print('--' * 10)

    command.run(['list', __file__ + ':10'])
    print('--' * 10)

    command.run(['list', 'os', '10'])
    command.proc.frame = sys._getframe()
    command.proc.setup()
    print('--' * 10)

    command.run(['list'])
    print('--' * 10)

    Mbreak  = import_relative('break', '.', 'pydbgr')
    brk_cmd = Mbreak.BreakCommand(d.core.processor)
    brk_cmd.run(['break'])
    command.run(['list', '.'])
    print('--' * 10)

    Mdisable     = import_relative('disable', '.', 'pydbgr')
    disable_cmd  = Mdisable.DisableCommand(d.core.processor)
    brk_cmd.run(['break'])
    disable_cmd.run(['disable', '2'])
    command.run(['list', '.'])
    print('--' * 10)

    command.run(['list', '10'])
    print('--' * 10)

    command.run(['list', '1000'])
    def foo():
        return 'bar'
    command.run(['list', 'foo'])
    print('--' * 10)

    command.run(['list', 'os.path'])
    print('--' * 10)
    command.run(['list', 'os.path', '15'])
    print('--' * 10)
    command.run(['list', 'os.path', '30', '3'])
    print('--' * 10)
    command.run(['list', 'os.path', '40', '50'])
    print('--' * 10)

    command.run(['list', os.path.abspath(__file__)+':3', '4'])
    print('--' * 10)
    command.run(['list', os.path.abspath(__file__)+':3', '12-10'])
    command.run(['list', 'os.path:5'])
    pass



#!/usr/bin/env python
"General integration tests"
import unittest

from import_relative import import_relative
Mhelper = import_relative('helper', '.')

class GeneralTests(unittest.TestCase):

    def test_macro(self):
        """Test macro and info macro"""
        result=Mhelper.run_debugger(testname='macro',
                                    dbgr_opts='--basename --highlight=plain',
                                    python_file='gcd.py')
        self.assertEqual(True, result, "debugger 'macro' command comparision")
        return
    pass

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
from import_relative import import_relative

# Our local modules
Mbase_cmd = import_relative('base_cmd', '.',  'pydbgr')
Mcmdfns   = import_relative('cmdfns',   '..', 'pydbgr')

class UpCommand(Mbase_cmd.DebuggerCommand):

    category      = 'stack'
    min_args      = 0
    max_args      = 1
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = True
    short_help    = 'Move frame in the direction of the caller of the last-selected frame'

    def run(self, args):
        """**up** [*count*]

Move the current frame up in the stack trace (to an older frame). 0 is
the most recent frame. If no count is given, move up 1.

See also `down` and `frame`."""

        if not self.proc.stack:
            self.errmsg("Program has no stack frame set.")
            return False
        if len(args) == 1:
            count = 1
        else:
            i_stack = len(self.proc.stack)
            count_str = args[1]
            count = Mcmdfns.get_an_int( self.errmsg, count_str,
                                        ("The 'up' command argument must eval to an" +
                                         " integer. Got: %s") % count_str,
                                        -i_stack, i_stack-1 )
            if count is None: return
            pass

        self.proc.adjust_frame(pos=count, absolute_pos=False)
        return False

if __name__ == '__main__':
    Mcmdproc     = import_relative('cmdproc', '..')
    Mdebugger    = import_relative('debugger', '...')
    d            = Mdebugger.Debugger()
    cp           = d.core.processor
    command = UpCommand(cp)
    command.run(['up'])

    def nest_me(cp, command, i):
        import inspect
        if i > 1:
            cp.curframe = inspect.currentframe()
            cp.stack, cp.curindex = Mcmdproc.get_stack(cp.curframe, None, None,
                                                       cp)
            command.run(['up'])
            print '-' * 10
            command.run(['up', '-2'])
            print '-' * 10
            command.run(['up', '-3'])
            print '-' * 10
        else:
            nest_me(cp, command, i+1)
        return

    cp.forget()
    nest_me(cp, command, 1)
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013 Rocky Bernstein
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from import_relative import import_relative
Mbase_cmd  = import_relative('base_cmd', top_name='pydbgr')
Mprint     = import_relative('print', '...lib', 'pydbgr')

class ExamineCommand(Mbase_cmd.DebuggerCommand):
    """**examine** *expr1* [*expr2* ...]
   
Examine value, type and object attributes of an expression. 

In contrast to normal Python expressions, expressions should not have
blanks which would cause shlex to see them as different tokens.

**Examples:**

    examine x+1   # ok
    examine x + 1 # not ok

See also `print`, `pp`, and `whatis`.
"""
        
    aliases       = ('x',)
    category      = 'data'
    min_args      = 1
    max_args      = None
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = True
    short_help    = "Examine value, type and object attributes of an expression"

    def run(self, args):
        for arg in args[1:]:
            s = Mprint.print_obj(arg, self.proc.curframe)
            self.msg(s)
            pass
        return 

    pass

if __name__ == '__main__':
    import inspect
    cmdproc     = import_relative('cmdproc', '..')
    debugger    = import_relative('debugger', '...')
    d           = debugger.Debugger()
    cp          = d.core.processor
    cp.curframe = inspect.currentframe()
    command = ExamineCommand(cp)
    command.run(['examine', '10'])
    me = []
    print('=' * 30)
    command.run(['examine', 'me'])
    print('=' * 30)
    command.run(['examine', 'Mbase_cmd.DebuggerCommand'])
    pass





#!/usr/bin/env python
'Unit test for pydbgr.exception'
import unittest
from import_relative import import_relative

Mexcept = import_relative('exception', '...pydbgr')

class TestDeguggerExcept(unittest.TestCase):

    def test_debugger_restart(self):
        try:
            raise Mexcept.DebuggerRestart(['a', 'b'])
        except Mexcept.DebuggerRestart:
            import sys
            self.assertEqual(['a', 'b'], sys.exc_value.sys_argv)
        else:
            self.assertFalse(True)
        pass
        return
        
if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2009, 2013 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from import_relative import import_relative
# Our local modules
Mbase_subcmd  = import_relative('base_subcmd', '..', 'pydbgr')
Mpp           = import_relative('pp', '....lib', 'pydbgr')

class InfoGlobals(Mbase_subcmd.DebuggerSubcommand):
    '''Show the debugged programs's global variables'''
    min_abbrev = 2
    need_stack = True
    short_help = '''Show the debugged programs's global variables'''

    def run(self, args):
        if not self.proc.curframe:
            self.errmsg("No frame selected.")
            return False
        var_names = list(self.proc.curframe.f_globals.keys())
        var_names.sort()
        for var_name in var_names:
            val = self.proc.getval(var_name)
            Mpp.pp(val, self.settings['width'], self.msg_nocr, self.msg,
                   prefix='%s =' % var_name)
            pass
        return False
    pass

if __name__ == '__main__':
    mock = import_relative('mock', '..')
    Minfo = import_relative('info', '..')
    Mdebugger = import_relative('debugger', '....')
    d, cp = mock.dbg_setup()
    i = Minfo.InfoCommand(cp)
    sub = InfoGlobals(i)
    import inspect
    cp.curframe = inspect.currentframe()
    sub.run([])



# Copyright (C) 2008, 2009, 2010 Rocky Bernstein <rocky@gnu.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Debugger packaging information"""

# To the extent possible we make this file look more like a
# configuration file rather than code like setup.py. I find putting
# configuration stuff in the middle of a function call in setup.py,
# which for example requires commas in between parameters, is a little
# less elegant than having it here with reduced code, albeit there
# still is some room for improvement.

# Things that change more often go here.
copyright   = '''Copyright (C) 2008-2010, 2013 Rocky Bernstein <rocky@gnu.org>.'''
classifiers =  ['Development Status :: 4 - Beta',
                'Environment :: Console',
                'Intended Audience :: Developers',
                'License :: OSI Approved :: GNU General Public License (GPL)',
                'Operating System :: OS Independent',
                'Programming Language :: Python',
                'Topic :: Software Development :: Debuggers',
                'Topic :: Software Development :: Libraries :: Python Modules',
                ]

# The rest in alphabetic order
author             = "Rocky Bernstein"
author_email       = "rocky@gnu.org"
ftp_url            = None
install_requires   = ['columnize >= 0.3.2', 
                      'import_relative >= 0.1.0',
                      'pyficache >= 0.2.01',
                      'pygments',
                      'tracer >= 0.2.2']
license            = 'GPL'
mailing_list       = 'python-debugger@googlegroups.com'
modname            = 'pydbgr'
namespace_packages = [
    'pydbgr', 
    'pydbgr.bwprocessor',
    'pydbgr.interfaces',
    'pydbgr.io',
    'pydbgr.lib',
    'pydbgr.processor',
    'pydbgr.processor.command',
    'pydbgr.processor.command.ipython_magic',
    'pydbgr.processor.command.info_subcmd',
    'pydbgr.processor.command.set_subcmd',
    'pydbgr.processor.command.show_subcmd'
]
packages           = namespace_packages
py_modules         = None
short_desc         = 'Modular Python Debugger'

import os
import os.path, sys
from import_relative import get_srcdir

# VERSION.py sets variable VERSION.
execfile(os.path.join(get_srcdir(), 'pydbgr', 'VERSION.py'))
version            = VERSION
web                = 'http://code.google.com/p/pydbgr/'

# tracebacks in zip files are funky and not debuggable
zip_safe = False 

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()
long_description   = ( read("README.txt") + '\n\n' +  read("NEWS") )

